/**
 * Copyright (C) 2012-2016  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client;

import java.util.HashMap;
import java.util.Map;

public class Translation
{

    public static final String ACTION = "ACTION";

    public static final String ASCENDING_ORDER_FOR_COLUMN = "ASCENDING_ORDER_FOR_COLUMN";

    public static final String CSV_FILE = "CSV_FILE";

    public static final String DATA_ERROR = "DATA_ERROR";

    public static final String DESCENDING_ORDER_FOR_COLUMN = "DESCENDING_ORDER_FOR_COLUMN";

    public static final String EXPORT = "EXPORT";

    public static final String ERROR = "ERROR";

    public static final String FILTERS = "FILTERS";

    public static final String FREEZE_COLUMN = "FREEZE_COLUMN";

    public static final String FREEZE_UNFREEZE_COLUMNS = "FREEZE_UNFREEZE_COLUMNS";

    public static final String HIDE_COLUMN = "HIDE_COLUMN";

    public static final String HIDE_UNHIDE_COLUMNS = "HIDE_UNHIDE_COLUMNS";

    public static final String IS_EMPTY = "IS_EMPTY";

    public static final String IS_NOT_EMPTY = "IS_NOT_EMPTY";

    public static final String MS_ACCESS_FILE = "MS_ACCESS_FILE";

    public static final String MS_EXCEL_FILE = "MS_EXCEL_FILE";

    public static final String NO_DATA_TO_DISPLAY = "NO_DATA_TO_DISPLAY";

    public static final String NO_NEW_DATA = "NO_NEW_DATA";

    public static final String QUICK_SEARCH = "QUICK_SEARCH";

    public static final String PDF_FILE = "PDF_FILE";

    public static final String PREPARING_DATA_PLEASE_WAIT = "PREPARING_DATA_PLEASE_WAIT";

    public static final String REMOVE_ALL_ORDERS = "REMOVE_ALL_ORDERS";

    public static final String REMOVE_FILTERS_FROM_COLUMN = "REMOVE_FILTERS_FROM_COLUMN";

    public static final String REMOVE_ORDER_FOR_COLUMN = "REMOVE_ORDER_FOR_COLUMN";

    public static final String SEARCH = "SEARCH";

    public static final String SEARCH_IN_COLUMNS = "SEARCH_IN_COLUMNS";

    public static final String SIZE_OF_THE_REQUESTED_DATA_IS_ZERO = "SIZE_OF_THE_REQUESTED_DATA_IS_ZERO";

    public static final String SOMETHING_IS_WRONG_IN_DATA_PREPARATION_PLEASE_CHECK_YOUR_DATA_SETTINGS =
        "SOMETHING_IS_WRONG_IN_DATA_PREPARATION_PLEASE_CHECK_YOUR_DATA_SETTINGS";

    public static final String UNFREEZE_COLUMN = "UNFREEZE_COLUMN";

    public static final String WHEN_TRANSMITTING_DATA_AN_ERROR_HAS_OCCURRED =
        "WHEN_TRANSMITTING_DATA_AN_ERROR_HAS_OCCURRED";

    private Map<String, String> translatedWords = new HashMap<>();

    public String get( String translatedWord )
    {
        if ( translatedWords.containsKey( translatedWord ) )
        {
            return translatedWords.get( translatedWord );
        }
        return null;
    }

    public void set( Map<String, String> translatedWords )
    {
        this.translatedWords.putAll( translatedWords );
    }

    public void processData( Map<String, String> translationData )
    {
        if ( translationData == null || translationData.keySet()
                                                       .isEmpty() )
        {
            return;
        }

        if ( translationData.containsKey( ACTION ) )
        {
            this.translatedWords.put( ACTION, translationData.get( ACTION ) );
        }

        if ( translationData.containsKey( ASCENDING_ORDER_FOR_COLUMN ) )
        {
            this.translatedWords.put( ASCENDING_ORDER_FOR_COLUMN, translationData.get( ASCENDING_ORDER_FOR_COLUMN )
                + " '_'" );
        }

        if ( translationData.containsKey( DATA_ERROR ) )
        {
            this.translatedWords.put( DATA_ERROR, translationData.get( DATA_ERROR ) );
        }

        if ( translationData.containsKey( DESCENDING_ORDER_FOR_COLUMN ) )
        {
            this.translatedWords.put( DESCENDING_ORDER_FOR_COLUMN, translationData.get( DESCENDING_ORDER_FOR_COLUMN )
                + " '_'" );
        }

        if ( translationData.containsKey( EXPORT ) )
        {
            this.translatedWords.put( EXPORT, translationData.get( EXPORT ) );
        }

        if ( translationData.containsKey( ERROR ) )
        {
            this.translatedWords.put( ERROR, translationData.get( ERROR ) );
        }

        if ( translationData.containsKey( CSV_FILE ) )
        {
            this.translatedWords.put( CSV_FILE, translationData.get( CSV_FILE ) );
        }

        if ( translationData.containsKey( MS_ACCESS_FILE ) )
        {
            this.translatedWords.put( MS_ACCESS_FILE, translationData.get( MS_ACCESS_FILE ) );
        }

        if ( translationData.containsKey( PDF_FILE ) )
        {
            this.translatedWords.put( PDF_FILE, translationData.get( PDF_FILE ) );
        }

        if ( translationData.containsKey( MS_EXCEL_FILE ) )
        {
            this.translatedWords.put( MS_EXCEL_FILE, translationData.get( MS_EXCEL_FILE ) );
        }

        if ( translationData.containsKey( FILTERS ) )
        {
            this.translatedWords.put( FILTERS, translationData.get( FILTERS ) );
        }

        if ( translationData.containsKey( FREEZE_COLUMN ) )
        {
            this.translatedWords.put( FREEZE_COLUMN, translationData.get( FREEZE_COLUMN ) + " '_'" );
        }

        if ( translationData.containsKey( FREEZE_UNFREEZE_COLUMNS ) )
        {
            this.translatedWords.put( FREEZE_UNFREEZE_COLUMNS, translationData.get( FREEZE_UNFREEZE_COLUMNS ) );
        }

        if ( translationData.containsKey( HIDE_COLUMN ) )
        {
            this.translatedWords.put( HIDE_COLUMN, translationData.get( HIDE_COLUMN ) + " '_'" );
        }

        if ( translationData.containsKey( HIDE_UNHIDE_COLUMNS ) )
        {
            this.translatedWords.put( HIDE_UNHIDE_COLUMNS, translationData.get( HIDE_UNHIDE_COLUMNS ) );
        }

        if ( translationData.containsKey( IS_EMPTY ) )
        {
            this.translatedWords.put( IS_EMPTY, translationData.get( IS_EMPTY ) );
        }

        if ( translationData.containsKey( IS_NOT_EMPTY ) )
        {
            this.translatedWords.put( IS_NOT_EMPTY, translationData.get( IS_NOT_EMPTY ) );
        }

        if ( translationData.containsKey( NO_DATA_TO_DISPLAY ) )
        {
            this.translatedWords.put( NO_DATA_TO_DISPLAY, translationData.get( NO_DATA_TO_DISPLAY ) );
        }

        if ( translationData.containsKey( NO_NEW_DATA ) )
        {
            this.translatedWords.put( NO_NEW_DATA, translationData.get( NO_NEW_DATA ) );
        }

        if ( translationData.containsKey( QUICK_SEARCH ) )
        {
            this.translatedWords.put( QUICK_SEARCH, translationData.get( QUICK_SEARCH ) );
        }

        if ( translationData.containsKey( PREPARING_DATA_PLEASE_WAIT ) )
        {
            this.translatedWords.put( PREPARING_DATA_PLEASE_WAIT, translationData.get( PREPARING_DATA_PLEASE_WAIT ) );
        }

        if ( translationData.containsKey( REMOVE_ALL_ORDERS ) )
        {
            this.translatedWords.put( REMOVE_ALL_ORDERS, translationData.get( REMOVE_ALL_ORDERS ) );
        }

        if ( translationData.containsKey( REMOVE_FILTERS_FROM_COLUMN ) )
        {
            this.translatedWords.put( REMOVE_FILTERS_FROM_COLUMN, translationData.get( REMOVE_FILTERS_FROM_COLUMN )
                + " '_'" );
        }

        if ( translationData.containsKey( REMOVE_ORDER_FOR_COLUMN ) )
        {
            this.translatedWords.put( REMOVE_ORDER_FOR_COLUMN, translationData.get( REMOVE_ORDER_FOR_COLUMN ) + " '_'" );
        }

        if ( translationData.containsKey( SEARCH ) )
        {
            this.translatedWords.put( SEARCH, translationData.get( SEARCH ) );
        }

        if ( translationData.containsKey( SEARCH_IN_COLUMNS ) )
        {
            this.translatedWords.put( SEARCH_IN_COLUMNS, translationData.get( SEARCH_IN_COLUMNS ) );
        }

        if ( translationData.containsKey( SIZE_OF_THE_REQUESTED_DATA_IS_ZERO ) )
        {
            this.translatedWords.put( SIZE_OF_THE_REQUESTED_DATA_IS_ZERO,
                                      translationData.get( SIZE_OF_THE_REQUESTED_DATA_IS_ZERO ) );
        }

        if ( translationData.containsKey( SOMETHING_IS_WRONG_IN_DATA_PREPARATION_PLEASE_CHECK_YOUR_DATA_SETTINGS ) )
        {
            this.translatedWords.put( SOMETHING_IS_WRONG_IN_DATA_PREPARATION_PLEASE_CHECK_YOUR_DATA_SETTINGS,
                                      translationData.get( SOMETHING_IS_WRONG_IN_DATA_PREPARATION_PLEASE_CHECK_YOUR_DATA_SETTINGS ) );
        }

        if ( translationData.containsKey( UNFREEZE_COLUMN ) )
        {
            this.translatedWords.put( UNFREEZE_COLUMN, translationData.get( UNFREEZE_COLUMN ) + " '_'" );
        }

        if ( translationData.containsKey( WHEN_TRANSMITTING_DATA_AN_ERROR_HAS_OCCURRED ) )
        {
            this.translatedWords.put( WHEN_TRANSMITTING_DATA_AN_ERROR_HAS_OCCURRED,
                                      translationData.get( WHEN_TRANSMITTING_DATA_AN_ERROR_HAS_OCCURRED ) );
        }
    }

}
