/**
 * Copyright (C) 2012-2016  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.components;

import static com.google.gwt.query.client.GQuery.$;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.dussan.vaadin.dtable.client.Translation;
import org.dussan.vaadin.dtable.client.VDTable;
import org.dussan.vaadin.dtable.client.components.base.TableComponent;

import com.google.gwt.query.client.GQuery;

public class Filter
    extends TableComponent
{

    private String mainId = "#v-dtable";

    private String frozenBodyId = "table#dt-frozen-body-table";

    private String bodyId = "table#dt-body-table";

    private String filterHtml = new String();

    private Map<Integer, List<String[]>> columnsFilter = new HashMap<>();

    public Filter( VDTable dTable )
    {
        super( dTable );
    }

    private void setFilter( String[] filters )
    {
        if ( filters != null && 0 < filters.length )
        {
            for ( int filter = 0; filter < filters.length; filter += 3 )
            {
                int column = Integer.parseInt( filters[filter] );
                List<String[]> tempFilters = columnsFilter.get( column );
                if ( tempFilters == null )
                {
                    tempFilters = new ArrayList<>();
                }
                tempFilters.add( new String[] { filters[filter + 1], filters[filter + 2] } );
                columnsFilter.put( column, tempFilters );
            }
        }
    }

    private void setFilterHtml( String filterHtml )
    {
        this.filterHtml = filterHtml;
    }

    private void showFilterHtml()
    {
        $( mainId + " div#dt-filter-info" ).remove();
        if ( getDTable().config()
                        .isFilterInfoEnabled()
            && !filterHtml.isEmpty() )
        {
            filterHtml = "<b><u>" + getDTable().translation()
                                               .get( Translation.FILTERS )
                + "</u></b> " + filterHtml;

            GQuery mainContainerWrapper = $( mainId ).find( "div#dt-main-container-wrapper" );
            GQuery tableTools = $( mainId ).find( "div#dt-table-tools-tray" + getDTable().getId() );
            if ( getDTable().config()
                            .isFilterInfoOnTop() )
            {
                if ( getDTable().tools()
                                .isVisible()
                    && getDTable().tools()
                                  .isOnTop() )
                {
                    tableTools.before( $( "<div/>" ).id( "dt-filter-info" )
                                                    .addClass( "dt-filter-info" )
                                                    .html( filterHtml ) );
                }
                else
                {
                    mainContainerWrapper.before( $( "<div/>" ).id( "dt-filter-info" )
                                                              .addClass( "dt-filter-info" )
                                                              .html( filterHtml ) );
                }
            }
            else if ( getDTable().config()
                                 .isFilterInfoOnBottom() )
            {
                if ( getDTable().tools()
                                .isVisible()
                    && getDTable().tools()
                                  .isOnBottom() )
                {
                    tableTools.after( $( "<div/>" ).id( "dt-filter-info" )
                                                   .addClass( "dt-filter-info" )
                                                   .html( filterHtml ) );
                }
                else
                {
                    mainContainerWrapper.after( $( "<div/>" ).id( "dt-filter-info" )
                                                             .addClass( "dt-filter-info" )
                                                             .html( filterHtml ) );
                }
            }
        }
        filterHtml = new String();
    }

    public void unmarkFilter( int column )
    {
        String filterId = "td [id$=':" + column + "']";
        $( getDTable().isFrozen( column ) ? frozenBodyId : bodyId ).children( "tr" )
                                                                   .children( filterId )
                                                                   .removeClass( "filters" );
    }

    public boolean isFilterEnabled( int column )
    {
        if ( getDTable().getAllRows() > 1 )
        {
            if ( !getDTable().isFrozen( column ) )
            {
                return getDTable().config()
                                  .isFilteredTableEnabled()
                    && getDTable().config()
                                  .isContextMenuForFilteredTableEnabled();
            }
            else
            {
                return getDTable().config()
                                  .isFilteredFrozenTableEnabled()
                    && getDTable().config()
                                  .isContextMenuForFilteredFrozenTableEnabled();
            }
        }
        return false;
    }

    public boolean isFiltered( int column )
    {
        return columnsFilter.containsKey( column );
    }

    public boolean filterOperatorExists( int column, String operator )
    {
        if ( isFiltered( column ) )
        {
            List<String[]> filters = columnsFilter.get( column );
            for ( String[] filter : filters )
            {
                if ( filter[0].equals( operator ) )
                {
                    return true;
                }
            }
        }
        return false;
    }

    public boolean filterExists( int column, String filter )
    {
        if ( isFiltered( column ) )
        {
            List<String[]> filters = columnsFilter.get( column );
            for ( String[] tempFilter : filters )
            {
                if ( tempFilter[1].equals( filter ) )
                {
                    return true;
                }
            }
        }
        return false;
    }

    public void newFilter( int column, int row, String operator, String filter, String cellValue )
    {
        if ( operator != null && !operator.isEmpty() )
        {
            getDTable().sendMessage( "newFilter", Integer.toString( column ) + "_::_" + Integer.toString( row )
                + "_::_" + operator + "_::_" + filter + "_::_" + cellValue, true );
        }
    }

    public void removeFilter( int column )
    {
        if ( isFiltered( column ) || getDTable().search()
                                                .isSearched( column ) )
        {
            unmarkFilter( column );
            getDTable().search()
                       .emptySearch( column );
            getDTable().sendMessage( "removeFilter", Integer.toString( column ), true );
        }
    }

    public void markFilter()
    {
        if ( ( ( getDTable().config()
                            .isFilteredFrozenTableEnabled()
            || getDTable().config()
                          .isFilteredTableEnabled() )
            && !columnsFilter.isEmpty() )
            || ( getDTable().config()
                            .isSearchEnabled()
                && !getDTable().search()
                               .isEmpty() ) )
        {
            List<Integer> columns = new ArrayList<>( columnsFilter.keySet() );
            columns.addAll( getDTable().search()
                                       .getSearchColumns() );
            Set<Integer> colSet = new HashSet<>( columns );
            columns = new ArrayList<>( colSet );
            Collections.sort( columns );

            for ( Integer column : columns )
            {
                String filterId = "td [id$=':" + column + "']";
                String id = getDTable().isFrozen( column ) ? frozenBodyId : bodyId;
                $( id ).children( "tr" )
                       .children( filterId )
                       .addClass( "filters" );
            }
        }
    }

    @Override
    public void setIds()
    {
        if ( !mainId.endsWith( "" + getDTable().getId() ) )
        {
            mainId += getDTable().getId();
            frozenBodyId += getDTable().getId();
            bodyId += getDTable().getId();
        }
    }

    public void processData( Map<String, String> data )
    {
        columnsFilter.clear();
        if ( data.containsKey( "newFilter" ) )
        {
            setFilter( VDTable.toStringArray( data.get( "newFilter" ) ) );
        }

        if ( data.containsKey( "filterHtml" ) )
        {
            setFilterHtml( data.get( "filterHtml" ) );
        }
    }

    @Override
    public void load()
    {
        setIds();
        setEventsHandlerEnabled();
    }

    @Override
    public void unload()
    {
        setEventsHandlerDisabled();
        $( mainId + " div#dt-filter-info" ).remove();
    }

    public void apply()
    {
        if ( ( ( getDTable().config()
                            .isFilteredFrozenTableEnabled()
            || getDTable().config()
                          .isFilteredTableEnabled() )
            && !columnsFilter.isEmpty() )
            || ( getDTable().config()
                            .isSearchEnabled()
                && !getDTable().search()
                               .isEmpty() ) )
        {
            showFilterHtml();
        }
        else
        {
            unload();
        }
    }

    @Override
    public void setEventsHandlerEnabled()
    {
    }

    @Override
    public void setEventsHandlerDisabled()
    {
    }

}
