/**
 * Copyright (C) 2012-2016  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.components;

import static com.google.gwt.query.client.GQuery.$;
import static com.google.gwt.query.client.plugins.Effects.Effects;

import org.dussan.vaadin.dtable.client.Translation;
import org.dussan.vaadin.dtable.client.VDTable;
import org.dussan.vaadin.dtable.client.components.base.TableComponent;

import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.Style.Display;
import com.google.gwt.query.client.Function;
import com.google.gwt.query.client.GQuery;
import com.google.gwt.query.client.css.CSS;
import com.google.gwt.query.client.css.Length;

public class LoadingMessage
    extends TableComponent
{

    private boolean showing = false;

    private String mainId = "div#v-dtable";

    private String bodyId = "table#dt-body-table";

    private String mainContainerWrapperId = "div#dt-main-container-wrapper";

    private String messageId = "div#dt-message";

    public LoadingMessage( VDTable dTable )
    {
        super( dTable );
    }

    @Override
    public void setIds()
    {
        if ( !mainId.endsWith( "" + getDTable().getId() ) )
        {
            mainId += getDTable().getId();
            bodyId += getDTable().getId();
            messageId += getDTable().getId();
        }
    }

    @Override
    public void load()
    {
        setIds();
        if ( !showing )
        {
            if ( getDTable().config()
                            .isLoadingMessageEnabled() )
            {
                $( messageId ).empty()
                              .append( $( "<div/>" ).id( "dt-message-wrapper" )
                                                    .addClass( "wrapper" )
                                                    .css( CSS.DISPLAY.with( Display.NONE ) )
                                                    .append( $( "<div/>" ).text( getDTable().translation()
                                                                                            .get( Translation.PREPARING_DATA_PLEASE_WAIT ) )
                                                                          .addClass( "context" ) ) );
            }
            else
            {
                unload();
            }
        }
        setEventsHandlerEnabled();
    }

    @Override
    public void unload()
    {
        $( messageId ).remove();
        setEventsHandlerDisabled();
    }

    public void show()
    {
        if ( getDTable().config()
                        .isLoadingMessageEnabled() )
        {
            showing = true;
            GQuery wrapper = $( mainId ).find( mainContainerWrapperId );
            $( messageId ).css( "display", "" )
                          .children( "div" )
                          .css( "display", "" )
                          .parent()
                          .css( CSS.LEFT.with( Length.px( 0 ) ) )
                          .css( CSS.TOP.with( Length.px( 0 ) ) )
                          .css( CSS.WIDTH.with( Length.px( wrapper.width() ) ) )
                          .css( CSS.HEIGHT.with( Length.px( wrapper.height() ) ) )
                          .css( CSS.ZINDEX.with( 10000 ) )
                          .as( Effects )
                          .animate( "opacity:1", getDTable().config()
                                                            .getLoadingMessageShowDelay() );
        }
    }

    public void sendMessage( final String eventType, final String eventMessage )
    {
        if ( getDTable().config()
                        .isLoadingMessageEnabled() )
        {
            $( bodyId ).parent()
                       .parent()
                       .parent()
                       .as( Effects )
                       .animate( "opacity:" + getDTable().config()
                                                         .getLoadingMessageOpacity(),
                                 getDTable().config()
                                            .getLoadingMessageShowDelay(),
                                 new Function()
                                 {
                                     public void f()
                                     {
                                         getDTable().sentMessage( eventType, eventMessage );
                                     }
                                 } );
        }
    }

    public void hide()
    {
        if ( getDTable().config()
                        .isLoadingMessageEnabled() )
        {
            GQuery wrapper = $( mainId ).find( mainContainerWrapperId );
            $( messageId ).as( Effects )
                          .animate( "opacity:0", getDTable().config()
                                                            .getLoadingMessageHideDelay(),
                                    new Function()
                                    {
                                        public void f( Element e )
                                        {
                                            showing = false;
                                            $( e ).css( CSS.DISPLAY.with( Display.NONE ) )
                                                  .css( CSS.LEFT.with( Length.px( 0 ) ) )
                                                  .css( CSS.TOP.with( Length.px( 0 ) ) )
                                                  .css( CSS.WIDTH.with( Length.px( 0 ) ) )
                                                  .css( CSS.HEIGHT.with( Length.px( 0 ) ) )
                                                  .css( CSS.ZINDEX.with( -1 ) );
                                            getDTable().setEventsHandlerEnabled();
                                        }
                                    } );
            wrapper.children( "table" )
                   .children( "tr" )
                   .as( Effects )
                   .animate( "opacity:1", getDTable().config()
                                                     .getLoadingMessageHideDelay() );
        }
        else
        {
            getDTable().setEventsHandlerEnabled();
        }
    }

    @Override
    public void setEventsHandlerEnabled()
    {
    }

    @Override
    public void setEventsHandlerDisabled()
    {
    }

}
