/**
 * Copyright (C) 2012-2016  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.components;

import static com.google.gwt.query.client.GQuery.$;
import static com.google.gwt.query.client.plugins.Effects.Effects;

import org.dussan.vaadin.dtable.client.Translation;
import org.dussan.vaadin.dtable.client.VDTable;
import org.dussan.vaadin.dtable.client.components.base.TableComponent;

import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.Style.Cursor;
import com.google.gwt.query.client.Function;
import com.google.gwt.query.client.GQuery;
import com.google.gwt.query.client.css.CSS;
import com.google.gwt.query.client.css.Length;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Event;

public class TableTools
    extends TableComponent
{

    private String absoluteHPosition = new String();

    private String absoluteVPosition = new String();

    private String actionId = "#dt-action";

    private String actionFreezeColumnsId = "#dt-actionFreezeColumns";

    private String actionHideColumnsId = "#dt-actionHideColumns";

    private String actionQuickSearchId = "#dt-actionQuickSearch";

    private String actionSearchId = "#dt-actionSearch";

    private String activeButton = null;

    private String exportId = "#dt-export";

    private String exportCsvId = "#dt-exportCsv";

    private String exportMdbId = "#dt-exportMdb";

    private String exportPdfId = "#dt-exportPdf";

    private String exportXlsId = "#dt-exportXls";

    private String mainId = "div#v-dtable";

    private String marginPosition = new String();

    private String paddingPosition = new String();

    private String tableToolsMenuId = "div#dt-table-tools-menu";

    private String tableToolsTrayId = "div#dt-table-tools-tray";

    private Element tableToolsMenuDiv = null;

    private Element tableToolsTrayDiv = null;

    private GQuery menuContextMenuEventHandler = null;

    private GQuery mouseClickEventHandler = null;

    private GQuery mouseOverEventHandler = null;

    private GQuery trayContextMenuEventHandler = null;

    public TableTools( VDTable dTable )
    {
        super( dTable );
    }

    private void activateMenuContextMenuEventHandler( boolean enable )
    {
        if ( enable && getDTable().config()
                                  .isTableToolsMenuEnabled() )
        {
            activateMenuContextMenuEventHandler( false );

            GQuery menu = $( tableToolsMenuId );
            menuContextMenuEventHandler = menu.on( "contextmenu", "div", new Function()
            {
                public boolean f( Event e )
                {
                    return false;
                }
            } );
        }
        else if ( !enable && menuContextMenuEventHandler != null )
        {
            menuContextMenuEventHandler.off( "contextmenu", "div" );
            menuContextMenuEventHandler = null;
        }
    }

    private void activateTrayContextMenuEventHandler( boolean enable )
    {
        if ( enable && getDTable().config()
                                  .isTableToolsTrayEnabled() )
        {
            activateTrayContextMenuEventHandler( false );

            GQuery tray = $( tableToolsTrayId );
            trayContextMenuEventHandler = tray.on( "contextmenu", "div", new Function()
            {
                public boolean f( Event e )
                {
                    return false;
                }
            } );
        }
        else if ( !enable && trayContextMenuEventHandler != null )
        {
            trayContextMenuEventHandler.off( "contextmenu", "div" );
            trayContextMenuEventHandler = null;
        }
    }

    private void activateMouseOverEventHandler( boolean activate )
    {
        if ( activate )
        {
            activateMouseOverEventHandler( false );

            GQuery tools = $( tableToolsTrayId );
            mouseOverEventHandler = tools.on( "mouseover", "div", new Function()
            {
                public boolean f( Event event )
                {
                    mouseover( event );
                    return false;
                }
            } )
                                         .on( "mouseout", "div", new Function()
                                         {
                                             public boolean f( Event event )
                                             {
                                                 mouseout( event );
                                                 return false;
                                             }
                                         } );
        }
        else if ( mouseOverEventHandler != null )
        {
            mouseOverEventHandler.off( "mouseover", "div" )
                                 .off( "mouseout", "div" );
            mouseOverEventHandler = null;
        }
    }

    private void activateMouseClickEventHandler( boolean activate )
    {
        if ( activate )
        {
            activateMouseClickEventHandler( false );

            GQuery main = $( mainId );
            mouseClickEventHandler = main.on( "click", tableToolsTrayId, new Function()
            {
                public boolean f( Event event )
                {
                    mouseClick( event );
                    return false;
                }
            } )
                                         .on( "dblclick", tableToolsTrayId, new Function()
                                         {
                                             public boolean f( Event event )
                                             {
                                                 return false;
                                             }
                                         } )
                                         .on( "click", tableToolsMenuId, new Function()
                                         {
                                             public boolean f( Event event )
                                             {
                                                 mouseClick( event );
                                                 return false;
                                             }
                                         } )
                                         .on( "dblclick", tableToolsMenuId, new Function()
                                         {
                                             public boolean f( Event event )
                                             {
                                                 return false;
                                             }
                                         } );
        }
        else if ( mouseClickEventHandler != null )
        {
            mouseClickEventHandler.off( "click", tableToolsTrayId )
                                  .off( "dblclick", tableToolsTrayId )
                                  .off( "click", tableToolsMenuId )
                                  .off( "dblclick", tableToolsMenuId );
            mouseClickEventHandler = null;
        }
    }

    private int getCompleteWidth()
    {
        int width = 0;
        for ( int element = 0; element < $( tableToolsTrayDiv ).children()
                                                               .size(); element++ )
        {
            width += $( tableToolsTrayDiv ).children()
                                           .eq( element )
                                           .outerWidth( true );
        }
        return width;
    }

    private int getCompleteHeight( boolean includeMargin )
    {
        if ( includeMargin )
        {
            return $( tableToolsTrayDiv ).outerHeight( true );
        }
        return $( tableToolsTrayDiv ).height();
    }

    private int getMarginSize( GQuery e )
    {
        String margin = e.css( marginPosition );
        if ( margin.endsWith( "px" ) )
        {
            return Math.abs( Integer.parseInt( margin.replace( "px", "" ) ) );
        }
        return 0;
    }

    private int getPaddinSize( GQuery e )
    {
        String padding = e.css( paddingPosition );
        if ( padding.endsWith( "px" ) )
        {
            return Math.abs( Integer.parseInt( padding.replace( "px", "" ) ) );
        }
        return 0;
    }

    private int getAbsoluteVPositionSize( GQuery e )
    {
        String absolute = e.css( absoluteVPosition );
        if ( absolute.endsWith( "px" ) )
        {
            return Integer.parseInt( absolute.replace( "px", "" ) );
        }
        return 0;
    }

    private boolean isActionButtonsEnabled()
    {
        return ( getDTable().config()
                            .isHideUnhideColumnsEnabled()
            && getDTable().config()
                          .isHideUnhideColumnsToolEnabled() )
            || ( getDTable().config()
                            .isFrozenTableEnabled()
                && getDTable().config()
                              .isFreezeUnfreezeColumnsEnabled()
                && getDTable().config()
                              .isFreezeUnfreezeColumnsToolEnabled() )
            || ( getDTable().config()
                            .isSearchEnabled()
                && getDTable().config()
                              .isSearchHideEnabled() )
            || ( getDTable().config()
                            .isQuickSearchEnabled()
                && getDTable().config()
                              .isQuickSearchHideEnabled() );
    }

    private boolean isExportButtonsEnabled()
    {
        return getDTable().exporter()
                          .isExportPdfFileEnabled()
            || getDTable().exporter()
                          .isExportMdbFileEnabled()
            || getDTable().exporter()
                          .isExportXlsFileEnabled()
            || getDTable().exporter()
                          .isExportCsvFileEnabled();
    }

    private void appendButton( String id, String idTitle, String clazz )
    {
        String hAbsolute = getCompleteWidth() + "px";
        String vAbsolute = ( -getCompleteHeight( true ) ) + "px";
        String animation = absoluteVPosition + ":+=" + getCompleteHeight( true ) + "px;";
        $( tableToolsTrayDiv ).append( $( "<div />" ).id( id )
                                                     .text( idTitle )
                                                     .addClass( clazz + getDTable().config()
                                                                                   .getTableToolsPosition() )
                                                     .css( absoluteHPosition, hAbsolute )
                                                     .css( absoluteVPosition, vAbsolute )
                                                     .delay( 50 )
                                                     .as( Effects )
                                                     .animate( animation, getDTable().config()
                                                                                     .getTableToolsTrayAnimationDuration(),
                                                               new Function()
                                                               {
                                                                   public void f( Element e )
                                                                   {
                                                                       int height = getCompleteHeight( false )
                                                                           - $( e ).outerHeight( true );
                                                                       $( e ).css( marginPosition, -height + "px" )
                                                                             .css( paddingPosition, height
                                                                                 + getPaddinSize( $( e ) ) + "px" )
                                                                             .css( absoluteVPosition, "" );
                                                                   }
                                                               } ) );
    }

    private void appendActionButtons()
    {
        if ( getDTable().config()
                        .isHideUnhideColumnsEnabled()
            && getDTable().config()
                          .isHideUnhideColumnsToolEnabled() )
        {
            appendButton( actionHideColumnsId.substring( 1 ), getDTable().translation()
                                                                         .get( Translation.HIDE_UNHIDE_COLUMNS ),
                          "action" );
        }

        if ( getDTable().config()
                        .isFrozenTableEnabled()
            && getDTable().config()
                          .isFreezeUnfreezeColumnsEnabled()
            && getDTable().config()
                          .isFreezeUnfreezeColumnsToolEnabled() )
        {
            appendButton( actionFreezeColumnsId.substring( 1 ), getDTable().translation()
                                                                           .get( Translation.FREEZE_UNFREEZE_COLUMNS ),
                          "action" );
        }

        if ( getDTable().config()
                        .isSearchEnabled()
            && getDTable().config()
                          .isSearchHideEnabled() )
        {
            appendButton( actionSearchId.substring( 1 ), getDTable().translation()
                                                                    .get( Translation.SEARCH_IN_COLUMNS ),
                          "action" );
        }

        if ( getDTable().config()
                        .isQuickSearchEnabled()
            && getDTable().config()
                          .isQuickSearchHideEnabled() )
        {
            appendButton( actionQuickSearchId.substring( 1 ), getDTable().translation()
                                                                         .get( Translation.QUICK_SEARCH ),
                          "action" );
        }
    }

    private void appendExportButtons()
    {
        if ( getDTable().exporter()
                        .isExportPdfFileEnabled() )
        {
            appendButton( exportPdfId.substring( 1 ), getDTable().translation()
                                                                 .get( Translation.PDF_FILE ),
                          "action" );
        }

        if ( getDTable().exporter()
                        .isExportMdbFileEnabled() )
        {
            appendButton( exportMdbId.substring( 1 ), getDTable().translation()
                                                                 .get( Translation.MS_ACCESS_FILE ),
                          "action" );
        }

        if ( getDTable().exporter()
                        .isExportXlsFileEnabled() )
        {
            appendButton( exportXlsId.substring( 1 ), getDTable().translation()
                                                                 .get( Translation.MS_EXCEL_FILE ),
                          "action" );
        }

        if ( getDTable().exporter()
                        .isExportCsvFileEnabled() )
        {
            appendButton( exportCsvId.substring( 1 ), getDTable().translation()
                                                                 .get( Translation.CSV_FILE ),
                          "action" );
        }
    }

    private void mouseover( Event event )
    {
        String animation = absoluteVPosition + ":+=" + getMarginSize( $( event ) ) + "px";
        if ( ( $( event ).is( actionId, exportId ) && activeButton == null ) || !$( event ).is( actionId, exportId ) )
        {
            $( event ).as( Effects )
                      .animate( animation, getDTable().config()
                                                      .getTableToolsTrayMouseOverAnimationDuration() );
        }
    }

    private void mouseout( Event event )
    {
        String animation = absoluteVPosition + ":-=" + getMarginSize( $( event ) ) + "px";
        if ( ( $( event ).is( actionId, exportId ) && activeButton == null ) || !$( event ).is( actionId, exportId ) )
        {
            $( event ).as( Effects )
                      .animate( animation, getDTable().config()
                                                      .getTableToolsTrayAnimationDuration(),
                                new Function()
                                {
                                    public void f( Element e )
                                    {
                                        $( e ).css( absoluteVPosition, "" );
                                    }
                                } );
        }
    }

    private void mouseClick( Event event )
    {
        // disable click events
        activateMouseClickEventHandler( false );

        final String id = DOM.eventGetTarget( event )
                             .getId();
        if ( id.equals( actionId.substring( 1 ) ) || id.equals( exportId.substring( 1 ) ) )
        {
            if ( activeButton == null )
            {
                String opacity = "opacity:0.4;filter:alpha(opacity=40);";
                $( event ).children( actionId, exportId )
                          .not( "#" + id )
                          .css( CSS.CURSOR.with( Cursor.DEFAULT ) )
                          .as( Effects )
                          .animate( opacity, getDTable().config()
                                                        .getTableToolsTrayAnimationDuration() );

                if ( id.equals( actionId.substring( 1 ) ) )
                {
                    appendActionButtons();
                }
                else if ( id.equals( exportId.substring( 1 ) ) )
                {
                    appendExportButtons();
                }
                activeButton = id;
            }
            else if ( id.equals( activeButton ) )
            {
                String opacity = "opacity:1.0;filter:alpha(opacity=100);";
                String animation = absoluteVPosition + ":-=" + getCompleteHeight( true ) + "px;";
                $( event ).children( actionId, exportId )
                          .not( "#" + id )
                          .as( Effects )
                          .animate( opacity, getDTable().config()
                                                        .getTableToolsTrayAnimationDuration(),
                                    new Function()
                                    {
                                        public void f( Element e )
                                        {
                                            $( e ).css( CSS.CURSOR.getCssName(), "" )
                                                  .css( "opacity", "" )
                                                  .css( "filter", "" );
                                        }
                                    } );
                $( event ).children( "div" )
                          .not( actionId, exportId )
                          .as( Effects )
                          .animate( animation, getDTable().config()
                                                          .getTableToolsTrayAnimationDuration(),
                                    new Function()
                                    {
                                        public void f( Element e )
                                        {
                                            $( e ).remove();
                                        }
                                    } );
                activeButton = null;
            }
        }
        else if ( id.startsWith( tableToolsMenuId.substring( 4 ) ) )
        {
            getDTable().tableToolsMenu()
                       .setPopupPosition( event.getClientX(), event.getClientY() );
            getDTable().tableToolsMenu()
                       .show();
        }
        else
        {
            // reset table tool's buttons
            String opacity = "opacity:1.0;filter:alpha(opacity=100);";
            $( event ).children( actionId, exportId )
                      .as( Effects )
                      .animate( opacity, getDTable().config()
                                                    .getTableToolsTrayAnimationDuration() )
                      .each( new Function()
                      {
                          public void f( Element e )
                          {
                              String animation = absoluteVPosition + ":-=" + getAbsoluteVPositionSize( $( e ) ) + "px;";
                              $( e ).as( Effects )
                                    .animate( animation, getDTable().config()
                                                                    .getTableToolsTrayAnimationDuration(),
                                              new Function()
                                              {
                                                  public void f( Element e )
                                                  {
                                                      $( e ).css( absoluteVPosition, "" )
                                                            .css( CSS.CURSOR.getCssName(), "" )
                                                            .css( "opacity", "" )
                                                            .css( "filter", "" );
                                                  }
                                              } );
                          }
                      } );

            String animation = absoluteVPosition + ":-=" + getCompleteHeight( true ) + "px;";
            $( event ).children( "div" )
                      .not( actionId, exportId )
                      .as( Effects )
                      .animate( animation, getDTable().config()
                                                      .getTableToolsTrayAnimationDuration(),
                                new Function()
                                {
                                    public void f( Element e )
                                    {
                                        $( e ).remove();
                                    }
                                } );
            activeButton = null;
        }

        // enable click events
        $( event ).delay( getDTable().config()
                                     .getTableToolsMouseClickActionDelay(),
                          new Function()
                          {
                              public void f()
                              {
                                  activateMouseClickEventHandler( true );

                                  // table tool's actions
                                  if ( id.equals( exportCsvId.substring( 1 ) ) )
                                  {
                                      getDTable().sendMessage( "export", "csv", false );
                                  }
                                  else if ( id.equals( exportMdbId.substring( 1 ) ) )
                                  {
                                      getDTable().sendMessage( "export", "mdb", false );
                                  }
                                  else if ( id.equals( exportPdfId.substring( 1 ) ) )
                                  {
                                      getDTable().sendMessage( "export", "pdf", false );
                                  }
                                  else if ( id.equals( exportXlsId.substring( 1 ) ) )
                                  {
                                      getDTable().sendMessage( "export", "xls", false );
                                  }
                                  else if ( id.equals( actionQuickSearchId.substring( 1 ) ) )
                                  {
                                      getDTable().quickSearch()
                                                 .toggle();
                                  }
                                  else if ( id.equals( actionSearchId.substring( 1 ) ) )
                                  {
                                      getDTable().search()
                                                 .toggle();
                                  }
                                  else if ( id.equals( actionFreezeColumnsId.substring( 1 ) ) )
                                  {
                                      getDTable().sendMessage( "columnsAction", "freeze", false );
                                  }
                                  else if ( id.equals( actionHideColumnsId.substring( 1 ) ) )
                                  {
                                      getDTable().sendMessage( "columnsAction", "hide", false );
                                  }
                              }
                          } );
    }

    @Override
    public void setIds()
    {
        if ( !mainId.endsWith( "" + getDTable().getId() ) )
        {
            mainId += getDTable().getId();
            tableToolsTrayId += getDTable().getId();
            tableToolsMenuId += getDTable().getId();

            actionId += getDTable().getId();
            actionQuickSearchId += getDTable().getId();
            actionSearchId += getDTable().getId();
            actionFreezeColumnsId += getDTable().getId();
            actionHideColumnsId += getDTable().getId();

            exportId += getDTable().getId();
            exportCsvId += getDTable().getId();
            exportMdbId += getDTable().getId();
            exportPdfId += getDTable().getId();
            exportXlsId += getDTable().getId();
        }
    }

    public boolean isVisible()
    {
        return !$( tableToolsTrayDiv ).isEmpty();
    }

    public boolean isOnTop()
    {
        return isVisible() && getDTable().config()
                                         .getTableToolsPosition()
                                         .contains( "top" );
    }

    public boolean isOnBottom()
    {
        return isVisible() && getDTable().config()
                                         .getTableToolsPosition()
                                         .contains( "bottom" );
    }

    @Override
    public void load()
    {
        setIds();

        if ( getDTable().config()
                        .isTableToolsTrayEnabled()
            && DOM.getElementById( tableToolsTrayId.substring( 4 ) ) == null )
        {
            tableToolsTrayDiv = DOM.createDiv();
            tableToolsTrayDiv.setId( tableToolsTrayId.substring( 4 ) );
            tableToolsTrayDiv.setClassName( "dt-table-tools" );

            if ( getDTable().config()
                            .getTableToolsPosition()
                            .contains( "left" ) )
            {
                absoluteHPosition = CSS.LEFT.getCssName();
            }
            else
            {
                absoluteHPosition = CSS.RIGHT.getCssName();
            }

            if ( getDTable().config()
                            .getTableToolsPosition()
                            .contains( "bottom" ) )
            {
                absoluteVPosition = CSS.TOP.getCssName();
                marginPosition = CSS.MARGIN_TOP.getCssName();
                paddingPosition = CSS.PADDING_TOP.getCssName();
                $( mainId ).find( "#dt-main-container-wrapper" )
                           .after( $( tableToolsTrayDiv ) );
            }
            else
            {
                absoluteVPosition = CSS.BOTTOM.getCssName();
                marginPosition = CSS.MARGIN_BOTTOM.getCssName();
                paddingPosition = CSS.PADDING_BOTTOM.getCssName();
                $( mainId ).find( "#dt-main-container-wrapper" )
                           .before( $( tableToolsTrayDiv ) );
            }

            if ( isActionButtonsEnabled() )
            {
                appendButton( actionId.substring( 1 ), getDTable().translation()
                                                                  .get( Translation.ACTION ),
                              "tool" );
            }

            if ( isExportButtonsEnabled() )
            {
                appendButton( exportId.substring( 1 ), getDTable().translation()
                                                                  .get( Translation.EXPORT ),
                              "tool" );
            }
        }
        if ( $( tableToolsTrayDiv ).children( actionId, exportId )
                                   .isEmpty() )
        {
            unload();
        }

        if ( getDTable().config()
                        .isTableToolsMenuEnabled() )
        {
            tableToolsMenuDiv = DOM.createDiv();
            $( mainId ).find( "#dt-header-wrapper" )
                       .append( $( tableToolsMenuDiv ).id( tableToolsMenuId.substring( 4 ) )
                                                      .addClass( "dt-table-tools-menu" )
                                                      .css( CSS.LINE_HEIGHT.with( Length.px( getDTable().config()
                                                                                                        .getHeaderRowHeight()
                                                          - 1 ) ) )
                                                      .height( getDTable().config()
                                                                          .getHeaderRowHeight()
                                                          - 1 ) );
            $( tableToolsMenuDiv ).append( $( "<div />" ).id( tableToolsMenuId.substring( 4 ) + ":0" ) );
        }
    }

    @Override
    public void unload()
    {
        setEventsHandlerDisabled();
        $( tableToolsTrayId ).remove();
        $( tableToolsMenuId ).remove();
    }

    @Override
    public void setEventsHandlerEnabled()
    {
        activateMenuContextMenuEventHandler( true );
        activateTrayContextMenuEventHandler( true );
        activateMouseOverEventHandler( true );
        activateMouseClickEventHandler( true );
    }

    @Override
    public void setEventsHandlerDisabled()
    {
        activateMenuContextMenuEventHandler( false );
        activateTrayContextMenuEventHandler( false );
        activateMouseOverEventHandler( false );
        activateMouseClickEventHandler( false );
    }

}
