/**
 * Copyright (C) 2012-2016  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.components.events;

import static com.google.gwt.query.client.GQuery.$;

import org.dussan.vaadin.dtable.client.components.QuickSearch;

import com.google.gwt.event.dom.client.KeyCodes;
import com.google.gwt.user.client.Event;

public class QuickSearchEvent
{

    public static boolean keyDown( QuickSearch quickSearch, Event event )
    {
        if ( !quickSearch.getDTable()
                         .sendingMessage() )
        {
            if ( event.getKeyCode() == KeyCodes.KEY_TAB )
            {
                $( event ).blur();
                return false;
            }
        }
        return true;
    }

    public static boolean keyUp( QuickSearch quickSearch, Event event )
    {
        if ( !quickSearch.getDTable()
                         .sendingMessage() )
        {
            if ( $( event ).id() != null && $( event ).id()
                                                      .endsWith( "" + quickSearch.getDTable()
                                                                                 .getId() ) )
            {
                String value = $( event ).val();

                if ( event.getKeyCode() == KeyCodes.KEY_ESCAPE )
                {
                    $( event ).blur();
                }
                else if ( event.getKeyCode() == KeyCodes.KEY_ENTER )
                {
                    if ( !quickSearch.getDTable()
                                     .config()
                                     .isQuickSearchImmediateEnabled()
                        && !quickSearch.isMessageSend() && !quickSearch.getFocusValue()
                                                                       .equals( value )
                        && ( !quickSearch.getEventValue()
                                         .equals( value )
                            || "".equals( quickSearch.getEventValue() ) ) )
                    {
                        quickSearch.setMessageSend( true );
                        quickSearch.setEventValue( value );
                        quickSearch.getDTable()
                                   .sendMessage( "quickSearch", value, true );
                    }

                    if ( quickSearch.getDTable()
                                    .config()
                                    .isQuickSearchImmediateEnabled() )
                    {
                        quickSearch.setEventValue( value );
                    }
                    quickSearch.setFocusValue( value );
                    $( event ).blur();
                }
                else if ( quickSearch.getDTable()
                                     .config()
                                     .isQuickSearchImmediateEnabled()
                    && event.getKeyCode() != KeyCodes.KEY_CTRL && event.getKeyCode() != KeyCodes.KEY_ALT
                    && event.getKeyCode() != KeyCodes.KEY_LEFT && event.getKeyCode() != KeyCodes.KEY_RIGHT
                    && event.getKeyCode() != KeyCodes.KEY_UP && event.getKeyCode() != KeyCodes.KEY_DOWN )
                {
                    if ( !quickSearch.getEventValue()
                                     .equals( value ) )
                    {
                        quickSearch.setMessageSend( true );
                        quickSearch.setEventValue( value );
                        quickSearch.getDTable()
                                   .sendMessage( "quickSearch", value, true );
                    }
                }
                return false;
            }
        }
        return true;
    }

    public static boolean focus( QuickSearch quickSearch, Event event )
    {
        if ( $( event ).id() != null && $( event ).id()
                                                  .endsWith( "" + quickSearch.getDTable()
                                                                             .getId() ) )
        {
            quickSearch.setFocusValue( $( event ).val() );
            quickSearch.setEventValue( $( event ).val() );
            quickSearch.setMessageSend( false );
        }
        return true;
    }

    public static boolean blur( QuickSearch quickSearch, Event event )
    {
        if ( $( event ).id() != null && $( event ).id()
                                                  .endsWith( "" + quickSearch.getDTable()
                                                                             .getId() ) )
        {
            String value = quickSearch.getFocusValue();
            if ( quickSearch.isMessageSend() && !value.equals( quickSearch.getEventValue() ) )
            {
                quickSearch.setEventValue( value );
                quickSearch.getDTable()
                           .sendMessage( "quickSearch", value, true );
            }
            $( event ).val( value );
        }
        return true;
    }

}
