/**
 * Copyright (C) 2012-2016  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.components.menu;

import java.util.Arrays;
import java.util.List;

public enum ContextMenuItems
{
    EQUAL( "EQUAL", "= _", "=" ),
    FILTER_REMOVE( "REMOVE_FILTER", "REMOVE_FILTERS_FROM_COLUMN", "" ),
    FREEZE( "FREEZE", "FREEZE_COLUMN", "" ),
    GREATER( "GREATER", "> _", ">" ),
    GREATER_EQUAL( "GREATER_EQUAL", ">= _", ">=" ),
    HIDE( "HIDE", "HIDE_COLUMN", "" ),
    IS_EMPTY( "IS_EMPTY", "IS_EMPTY", "=" ),
    IS_NOT_EMPTY( "IS_NOT_EMPTY", "IS_NOT_EMPTY", "<>" ),
    LESS( "LESS", "< _", "<" ),
    LESS_EQUAL( "LESS_EQUAL", "<= _", "<=" ),
    LIKE( "LIKE", "LIKE '_'", "LIKE" ),
    LIKE_LEFT( "LIKE_LEFT", "LIKE '%_'", "LIKE" ),
    LIKE_LEFT_RIGHT( "LIKE_LEFT_RIGHT", "LIKE '%_%'", "LIKE" ),
    LIKE_RIGHT( "LIKE_RIGHT", "LIKE '_%'", "LIKE" ),
    NONE( "NONE", "_", "" ),
    NOT_EQUAL( "NOT_EQUAL", "<> _", "<>" ),
    NOT_LIKE( "NOT_LIKE", "NOT LIKE '_'", "NOT LIKE" ),
    NOT_LIKE_LEFT( "NOT_LIKE_LEFT", "NOT LIKE '%_'", "NOT LIKE" ),
    NOT_LIKE_LEFT_RIGHT( "NOT_LIKE_LEFT_RIGHT", "NOT LIKE '%_%'", "NOT LIKE" ),
    NOT_LIKE_RIGHT( "NOT_LIKE_RIGHT", "NOT LIKE '_%'", "NOT LIKE" ),
    ORDER_ASCENDING( "ASC", "ASCENDING_ORDER_FOR_COLUMN", "" ),
    ORDER_DESCENDING( "DESC", "DESCENDING_ORDER_FOR_COLUMN", "" ),
    ORDER_REMOVE( "REMOVE_ORDER", "REMOVE_ORDER_FOR_COLUMN", "" ),
    ORDER_REMOVE_ALL( "REMOVE_ORDER_ALL", "REMOVE_ALL_ORDERS", "" ),
    UNFREEZE( "UNFREEZE", "UNFREEZE_COLUMN", "" );

    private final List<String> item;

    private ContextMenuItems( String... itemValues )
    {
        this.item = Arrays.asList( itemValues );
    }

    public List<String> getValues()
    {
        return item;
    }

    @Override
    public String toString()
    {
        return item.get( 1 );
    }

    public String id()
    {
        return item.get( 0 );
    }

    public String description()
    {
        return item.get( 1 );
    }

    public String operator()
    {
        return item.get( 2 );
    }

    public static ContextMenuItems item( String itemValue )
    {
        for ( ContextMenuItems item : ContextMenuItems.values() )
        {
            if ( item.getValues()
                     .contains( itemValue ) )
            {
                return item;
            }
        }
        return ContextMenuItems.NONE;
    }

}
