/**
 * Copyright (C) 2012-2016  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.client.components.menu;

import static com.google.gwt.query.client.GQuery.$;

import org.dussan.vaadin.dtable.client.Translation;
import org.dussan.vaadin.dtable.client.VDTable;

import com.google.gwt.dom.client.Element;
import com.google.gwt.event.dom.client.KeyCodes;
import com.google.gwt.query.client.Function;
import com.google.gwt.query.client.css.CSS;
import com.google.gwt.query.client.css.Length;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.Event.NativePreviewEvent;
import com.google.gwt.user.client.Window;
import com.vaadin.client.ui.VOverlay;

public class TableToolsMenu
    extends VOverlay
{

    private String actionId = "dt-menu-actionId";

    private String actionQuickSearchId = "dt-menu-actionQuickSearch";

    private String actionSearchId = "dt-menu-actionSearch";

    private String actionFreezeColumnsId = "dt-menu-actionFreezeColumns";

    private String actionHideColumnsId = "dt-menu-actionHideColumns";

    private String exportId = "dt-menu-exportId";

    private String exportCsvId = "dt-menu-exportCsv";

    private String exportMdbId = "dt-menu-exportMdb";

    private String exportPdfId = "dt-menu-exportPdf";

    private String exportXlsId = "dt-menu-exportXls";

    private VDTable dTable = null;

    private Element tableToolsMenu = null;

    public TableToolsMenu( VDTable dTable )
    {
        this.dTable = dTable;

        getElement().setClassName( "dt-menu" );
        setAutoHideEnabled( true );

        if ( 0 < DOM.getChildCount( getElement() ) )
        {
            getElement().getFirstChildElement()
                        .removeFromParent();
        }

        sinkEvents( Event.ONCLICK | Event.ONCONTEXTMENU );
    }

    private VDTable getDTable()
    {
        return dTable;
    }

    private boolean isActionMenuItemsEnabled()
    {
        return ( getDTable().config()
                            .isHideUnhideColumnsEnabled()
            && getDTable().config()
                          .isHideUnhideColumnsToolEnabled() )
            || ( getDTable().config()
                            .isFrozenTableEnabled()
                && getDTable().config()
                              .isFreezeUnfreezeColumnsEnabled()
                && getDTable().config()
                              .isFreezeUnfreezeColumnsToolEnabled() )
            || ( getDTable().config()
                            .isSearchEnabled()
                && getDTable().config()
                              .isSearchHideEnabled() )
            || ( getDTable().config()
                            .isQuickSearchEnabled()
                && getDTable().config()
                              .isQuickSearchHideEnabled() );
    }

    private boolean isExportMenuItemsEnabled()
    {
        return getDTable().exporter()
                          .isExportPdfFileEnabled()
            || getDTable().exporter()
                          .isExportMdbFileEnabled()
            || getDTable().exporter()
                          .isExportXlsFileEnabled()
            || getDTable().exporter()
                          .isExportCsvFileEnabled();
    }

    private void appendMenuItem( String id, String title, String clazz )
    {
        Element item = DOM.createDiv();
        item.setId( id );
        item.setInnerText( title );
        if ( clazz != null )
        {
            item.addClassName( clazz );
        }
        tableToolsMenu.appendChild( item );
    }

    private void appendActionMenuItems()
    {
        if ( isActionMenuItemsEnabled() )
        {
            appendMenuItem( actionId, getDTable().translation()
                                                 .get( Translation.ACTION ),
                            "group" );

            if ( getDTable().config()
                            .isHideUnhideColumnsEnabled()
                && getDTable().config()
                              .isHideUnhideColumnsToolEnabled() )
            {
                appendMenuItem( actionHideColumnsId, getDTable().translation()
                                                                .get( Translation.HIDE_UNHIDE_COLUMNS ),
                                null );
            }

            if ( getDTable().config()
                            .isFrozenTableEnabled()
                && getDTable().config()
                              .isFreezeUnfreezeColumnsEnabled()
                && getDTable().config()
                              .isFreezeUnfreezeColumnsToolEnabled() )
            {
                appendMenuItem( actionFreezeColumnsId, getDTable().translation()
                                                                  .get( Translation.FREEZE_UNFREEZE_COLUMNS ),
                                null );
            }

            if ( getDTable().config()
                            .isSearchEnabled()
                && getDTable().config()
                              .isSearchHideEnabled() )
            {
                appendMenuItem( actionSearchId, getDTable().translation()
                                                           .get( Translation.SEARCH_IN_COLUMNS ),
                                null );
            }

            if ( getDTable().config()
                            .isQuickSearchEnabled()
                && getDTable().config()
                              .isQuickSearchHideEnabled() )
            {
                appendMenuItem( actionQuickSearchId, getDTable().translation()
                                                                .get( Translation.QUICK_SEARCH ),
                                null );
            }
        }
    }

    private void appendExportMenuItems()
    {
        if ( isExportMenuItemsEnabled() )
        {
            appendMenuItem( exportId, getDTable().translation()
                                                 .get( Translation.EXPORT ),
                            "group" );

            if ( getDTable().exporter()
                            .isExportPdfFileEnabled() )
            {
                appendMenuItem( exportPdfId, getDTable().translation()
                                                        .get( Translation.PDF_FILE ),
                                null );
            }

            if ( getDTable().exporter()
                            .isExportMdbFileEnabled() )
            {
                appendMenuItem( exportMdbId, getDTable().translation()
                                                        .get( Translation.MS_ACCESS_FILE ),
                                null );
            }

            if ( getDTable().exporter()
                            .isExportXlsFileEnabled() )
            {
                appendMenuItem( exportXlsId, getDTable().translation()
                                                        .get( Translation.MS_EXCEL_FILE ),
                                null );
            }

            if ( getDTable().exporter()
                            .isExportCsvFileEnabled() )
            {
                appendMenuItem( exportCsvId, getDTable().translation()
                                                        .get( Translation.CSV_FILE ),
                                null );
            }
        }
    }

    @Override
    public void hide()
    {
        $( "#dt-menu-glass" ).remove();
        super.hide();
    }

    @Override
    protected void onPreviewNativeEvent( NativePreviewEvent event )
    {
        switch ( event.getTypeInt() )
        {
            case Event.ONKEYDOWN:
                if ( event.getNativeEvent()
                          .getKeyCode() == KeyCodes.KEY_ESCAPE )
                {
                    hide();
                }
                break;
            default:
                break;
        }
        super.onPreviewNativeEvent( event );
    }

    @Override
    public void onBrowserEvent( Event event )
    {
        event.stopPropagation();

        switch ( DOM.eventGetType( event ) )
        {
            case Event.ONCLICK:
                final String menuItemId = DOM.eventGetTarget( event )
                                             .getId();
                if ( !menuItemId.equals( actionId ) && !menuItemId.equals( exportId ) )
                {
                    hide();
                    $( event ).delay( getDTable().config()
                                                 .getTableToolsMouseClickActionDelay(),
                                      new Function()
                                      {
                                          public void f()
                                          {
                                              if ( menuItemId.equals( actionFreezeColumnsId ) )
                                              {
                                                  getDTable().sendMessage( "columnsAction", "freeze", false );
                                              }
                                              else if ( menuItemId.equals( actionHideColumnsId ) )
                                              {
                                                  getDTable().sendMessage( "columnsAction", "hide", false );
                                              }
                                              else if ( menuItemId.equals( actionQuickSearchId ) )
                                              {
                                                  getDTable().quickSearch()
                                                             .toggle();
                                              }
                                              else if ( menuItemId.equals( actionSearchId ) )
                                              {
                                                  getDTable().search()
                                                             .toggle();
                                              }
                                              else if ( menuItemId.equals( exportCsvId ) )
                                              {
                                                  getDTable().sendMessage( "export", "csv", false );
                                              }
                                              else if ( menuItemId.equals( exportMdbId ) )
                                              {
                                                  getDTable().sendMessage( "export", "mdb", false );
                                              }
                                              else if ( menuItemId.equals( exportPdfId ) )
                                              {
                                                  getDTable().sendMessage( "export", "pdf", false );
                                              }
                                              else if ( menuItemId.equals( exportXlsId ) )
                                              {
                                                  getDTable().sendMessage( "export", "xls", false );
                                              }
                                          }
                                      } );
                }
            case Event.ONCONTEXTMENU:
                event.preventDefault();
                break;
            default:
                break;
        }
    }

    @Override
    public void show()
    {
        if ( isActionMenuItemsEnabled() || isExportMenuItemsEnabled() )
        {
            if ( tableToolsMenu != null )
            {
                tableToolsMenu.removeFromParent();
            }

            // glass layer to prevent browser's events
            $( "<div />" ).id( "dt-menu-glass" )
                          .addClass( "dt-menu-glass" )
                          .css( CSS.WIDTH.with( Length.px( Window.getClientWidth() ) ) )
                          .css( CSS.HEIGHT.with( Length.px( Window.getClientHeight() ) ) )
                          .on( "click", new Function()
                          {
                              public void f( Element e )
                              {
                                  $( e ).remove();
                              }
                          } )
                          .appendTo( $( "body" ) );

            // table tools menu
            tableToolsMenu = DOM.createDiv();
            getElement().appendChild( tableToolsMenu );
            appendActionMenuItems();
            appendExportMenuItems();
            super.show();

            int popupLeft = ( getAbsoluteLeft() + getOffsetWidth() + 5 ) <= Window.getClientWidth() ? getPopupLeft()
                            : ( Window.getClientWidth() - getOffsetWidth() - 5 );
            int popupTop = ( getAbsoluteTop() + getOffsetHeight() + 5 ) <= Window.getClientHeight() ? getPopupTop()
                            : ( Window.getClientHeight() - getOffsetHeight() - 5 );

            setPopupPosition( popupLeft, popupTop );
        }
    }

}
