/**
 * Copyright (C) 2012-2016  Dušan Vejnovič  <vaadin@dussan.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dussan.vaadin.dtable.processors;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.dussan.vaadin.dtable.properties.filter.Filter;

public class FilterProcessor
{

    public static Filter process( String[] newFilter, Filter filter, Object originalValue, String header )
    {
        if ( newFilter != null && newFilter.length > 0 )
        {
            Integer column = Integer.parseInt( newFilter[0] );
            List<String> operators = new ArrayList<>();
            List<String> values = new ArrayList<>();
            List<String> originalValues = new ArrayList<>();

            if ( filter != null )
            {
                operators.addAll( Arrays.asList( filter.getOperators() ) );
                values.addAll( Arrays.asList( filter.getValues() ) );
                originalValues.addAll( Arrays.asList( filter.getOriginalValues() ) );
            }

            String newOperator = newFilter[2];
            String newValue = newFilter.length < 5 ? "" : newFilter[4].replace( "&nbsp;", "" )
                                                                      .trim();
            String newOriginalValue = originalValue == null ? new String() : originalValue.toString();
            if ( newValue.isEmpty() && ( newOperator.equals( "=" ) || newOperator.equals( "<>" ) ) )
            {
                newValue = "";
                newOriginalValue = "";
            }
            else if ( newOperator.endsWith( "LIKE" ) )
            {
                newValue = newFilter[3].substring( newOperator.length() )
                                       .replace( "'", "" )
                                       .trim();
                newOriginalValue = newValue;
            }
            operators.add( newOperator );
            values.add( newValue );
            originalValues.add( newOriginalValue );

            return new Filter( column, header, operators.toArray( new String[] {} ), values.toArray( new String[] {} ),
                               originalValues.toArray( new String[] {} ) );
        }
        return null;
    }

}
